
local rematch = Rematch
local settings

-- the scrollable options
-- when adding options:
-- 1. make the name of the option (2nd field) precisely the same as the setting.name
-- 2. add a REMATCH_OPT_name_LABEL for its localized text
-- 3. create a ReflectSetting.name function if needed (optional)
rematch.optionsList = {
	{ "header", REMATCH_OPT_TARGETING_HEADER },
	{ "check", "AutoShow" },
	{ "check", "AutoLoad" },
	{ "check", "AutoMouseover", "AutoLoad" },
	{ "check", "AutoLoadAlways", "AutoLoad" },
	{ "header", REMATCH_OPT_WINDOW_HEADER },
	{ "check", "LargeWindow" },
	{ "check", "LockWindow" },
	{ "check", "LockDrawer" },
	{ "check", "ShowSidePanel" },
	{ "check", "StayForBattle" },
	{ "check", "PrioritizeBrowser" },
	{ "header", REMATCH_OPT_LOADING_HEADER },
	{ "check", "EmptyMissing" },
	{ "check", "KeepSummoned" },
	{ "check", "KeepLeveling" },
	{ "header", REMATCH_OPT_LEVELING_HEADER },
	{ "check", "AutoConfirmFill" },
	{ "check", "KeepCurrentOnSort" },
	{ "check", "KeepQueueSort" },
	{ "check", "HidePetToast" },
	{ "header", REMATCH_OPT_BROWSER_HEADER },
	{ "check", "WideSearchBar" },
	{ "check", "UseTypeBar" },
	{ "check", "OnlyBattlePets" },
	{ "header", MISCELLANEOUS },
	{ "check", "JumpToTeam" },
	{ "check", "AutoConfirm" },
	{ "check", "BestOfSpecies" },
	{ "check", "DisableShare" },
	{ "check", "HideTooltips" },
	{ "check", "HideWarnings", "HideTooltips" },
	{ "check", "UseMinimapButton" },
	{ "check", "HideJournalButton" },
	{ "button", "BindingsButton" },
}

rematch.ReflectSetting = {} -- table of functions to run to reflect changes for defined settings
rematch.OptionsListButtons = {} -- table of functions to run when a "button" option is clicked

function rematch:InitializeOptions()
	settings = RematchSettings
	local options = rematch.drawer.options
	options.header.text:SetText(REMATCH_OPTIONS)
	options.version:SetText("Rematch version "..GetAddOnMetadata("Rematch","Version"))

	local scrollFrame = rematch.drawer.options.list.scrollFrame
	scrollFrame.update = rematch.UpdateOptionsList
	scrollFrame.stepSize = 40
	HybridScrollFrame_CreateButtons(scrollFrame,"RematchOptionsListTemplate")
	local scrollBar = scrollFrame.scrollBar
	scrollBar.doNotHide = true
	scrollBar.trackBG:SetPoint("TOPLEFT",-2,15)
	scrollBar.trackBG:SetPoint("BOTTOMRIGHT",2,-15)
	scrollBar.trackBG:SetTexture("Interface\\ChatFrame\\ChatFrameBackground")
	scrollBar.trackBG:SetGradientAlpha("HORIZONTAL",.125,.125,.125,1,.05,.05,.05,1)

	-- go through each checkbutton, set its text and run its reflect if it has one
	local list = rematch.optionsList
	for i=1,#list do
		if rematch.ReflectSetting[list[i][2]] then
			rematch.ReflectSetting[list[i][2]]()
		end
	end
end

function rematch:OptionsOnShow()
	rematch.saved.header:Hide()
	rematch.current.header:Hide()
	rematch:UpdateOptionsList()
end

function rematch:OptionsOnHide()
	rematch.saved.header:Show()
	rematch.current.header:Show()
end

function rematch:RematchOptionsButton()
	rematch.drawer.options:Show()
end

function rematch:RematchOptionsOkayButton()
	rematch.drawer.options:Hide()
end

function rematch.ReflectSetting.AutoLoad()
	rematch.ReflectSetting.AutoMouseover() -- update whether UPDATE_UNIT_MOUSEOVER should be registered
	rematch.sidePanel.autoLoad:SetChecked(settings.AutoLoad)
end

function rematch.ReflectSetting.AutoMouseover()
	if settings.AutoMouseover and settings.AutoLoad then
		rematch:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
	else
		rematch:UnregisterEvent("UPDATE_MOUSEOVER_UNIT")
	end
end

function rematch.ReflectSetting.LargeWindow()
	-- instead of a straight SetScale, we'll adjust position so it stays at the same
	-- relative position on the screen, growing out/in from the bottomleft corner
	local oldScale = rematch:GetEffectiveScale()
	settings.XPos = rematch:GetLeft()
	settings.YPos = rematch:GetBottom()
	rematch:SetScale(settings.LargeWindow and 1.25 or 1.0)
	local newScale = rematch:GetEffectiveScale()
	settings.XPos = (settings.XPos*oldScale)/newScale
	settings.YPos = (settings.YPos*oldScale)/newScale
	rematch:ClearAllPoints()
	rematch:SetPoint("BOTTOMLEFT",settings.XPos,settings.YPos)
	RematchFloatingPetCard:SetScale(settings.LargeWindow and 1.2 or 1)
end

function rematch.ReflectSetting.BestOfSpecies()
	rematch:SetupBestOfSpecies()
end

function rematch:ToggleAutoLoad()
	settings.AutoLoad = not settings.AutoLoad
	if rematch.drawer.options:IsVisible() then
		rematch:UpdateOptionsList()
	else
		print(REMATCH_AUTOLOAD_IS_NOW,settings.AutoLoad and REMATCH_AUTOLOAD_ENABLED or REMATCH_AUTOLOAD_DISABLED)
	end
end

function rematch:UpdateOptionsList()
	local list = rematch.optionsList
	local numData = #list
	local scrollFrame = rematch.drawer.options.list.scrollFrame
	local offset = HybridScrollFrame_GetOffset(scrollFrame)
	local buttons = scrollFrame.buttons
	for i=1, #buttons do
		local index = i + offset
		local button = buttons[i]
		button.index = nil
		button:Hide()
		if ( index <= numData) then
			button.index = index
			local label = _G["REMATCH_OPT_"..list[index][2].."_LABEL"]
			button.name:ClearAllPoints()
			if list[index][1]=="header" then
				button.name:SetText(label and label or list[index][2])
				button.name:SetTextColor(1,.82,0)
				button.name:SetPoint("CENTER")
				button.back:Show()
				button.check:Hide()
				button.button:Hide()
			elseif list[index][1]=="check" then
				button.name:SetText(label and label or list[index][2])
				button.back:Hide()
				button.check:Show()
				button.button:Hide()
				button.check:SetPoint("LEFT",list[index][3] and 16 or 6,0)
				button.name:SetPoint("LEFT",button.check,"RIGHT",2,0)
				if list[index][3] and not settings[list[index][3]] then
					button.check:SetEnabled(0) -- disable dependant option if parent unchecked
					button.name:SetTextColor(.5,.5,.5)
				else
					button.check:SetEnabled(1)
					button.name:SetTextColor(1,1,1)
				end
				button.check:SetChecked(settings[list[index][2]])
			elseif list[index][1]=="button" then
				button.name:SetText("")
				button.button:SetText(label and label or list[index][2])
				button.back:Hide()
				button.check:Hide()
				button.button:Show()
			end
			button:Show()
		end
	end
	HybridScrollFrame_Update(scrollFrame,20*numData,20)
end

function Rematch:OptionsCheckOnEnter()
	local index = self:GetParent().index
	local list = rematch.optionsList
	if index and list[index] and list[index][2] then
		local tooltip = _G["REMATCH_OPT_"..list[index][2].."_DESC"]
		if tooltip then
			rematch.drawer.options.tooltip.text:SetText(tooltip)
		end
	end
end

function Rematch:OptionsCheckOnLeave()
	rematch.drawer.options.tooltip.text:SetText("")
end

function rematch:OptionsCheckOnClick()
	local index = self:GetParent().index
	local list = rematch.optionsList
	if index and list[index] and list[index][2] then
		local setting = list[index][2]
		settings[setting] = self:GetChecked()
		rematch:UpdateOptionsList()
		if rematch.ReflectSetting[setting] then
			rematch:HideDropDown()
			rematch.ReflectSetting[setting]()
		end
	end
end

function rematch:OptionsListButtonOnClick()
	local index = self:GetParent().index
	local list = rematch.optionsList
	if index and list[index] and list[index][2] and rematch.OptionsListButtons[list[index][2]] then
		rematch.OptionsListButtons[list[index][2]]()
	end
end

-- jumps to Rematch section in default key binding interface
function rematch.OptionsListButtons.BindingsButton()
	local offset
	local numBindings = GetNumBindings()
	for i=1,numBindings do
		if GetBinding(i)=="HEADER_REMATCH" then
			offset=i
			break
		end
	end
	if offset then
		LoadAddOn("Blizzard_BindingUI")
		ShowUIPanel(KeyBindingFrame)
		local limit = numBindings-KEY_BINDINGS_DISPLAYED
		offset = offset>limit and limit or max(0,offset-1)
		FauxScrollFrame_OnVerticalScroll(KeyBindingFrameScrollFrame, offset*KEY_BINDING_HEIGHT, KEY_BINDING_HEIGHT, KeyBindingFrame_Update)
  end
end

--[[ Minimap Button ]]

function rematch:CreateMinimapButton()
	if RematchMinimapButton then
		return
	end
	local button = CreateFrame("Button","RematchMinimapButton",Minimap)
	button:SetSize(31,31)
	button:SetToplevel(true)
	button:SetFrameStrata("MEDIUM")
	button:SetFrameLevel(button:GetFrameLevel()+3)
	button:RegisterForClicks("AnyUp")
	button:SetHighlightTexture("Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight")
	button.icon = button:CreateTexture(nil,"BACKGROUND")
	button.icon:SetTexture("Interface\\Icons\\PetJournalPortrait")
	button.icon:SetSize(20,20)
	button.icon:SetPoint("CENTER")
	local border = button:CreateTexture(nil,"OVERLAY")
	border:SetTexture("Interface\\Minimap\\MiniMap-TrackingBorder")
	border:SetSize(53,53)
	border:SetPoint("TOPLEFT")
	button:SetScript("OnMouseDown",rematch.MinimapButtonOnMouseDown)
	button:SetScript("OnMouseUp",rematch.MinimapButtonOnMouseUp)
	button:SetScript("OnClick",rematch.Toggle)
	button:SetScript("OnEnter",rematch.ToggleTooltip)
	button:SetScript("OnLeave",rematch.ToggleTooltipHide)
	button:RegisterForDrag("LeftButton")
	button:SetScript("OnDragStart",rematch.MinimapButtonOnDragStart)
	button:SetScript("OnDragStop",rematch.MinimapButtonOnDragStop)
end

function rematch:MinimapButtonOnMouseDown()
	self.icon:SetPoint("CENTER",1,-1)
	self.icon:SetVertexColor(.75,.75,.75)
end

function rematch:MinimapButtonOnMouseUp()
	self.icon:SetPoint("CENTER")
	self.icon:SetVertexColor(1,1,1)
end

function rematch:MinimapButtonOnDragStart()
	self.update = self.update or CreateFrame("Frame",nil,self)
	self.update:SetScript("OnUpdate",rematch.MinimapButtonDragUpdate)
end

function rematch:MinimapButtonOnDragStop()
	rematch.MinimapButtonOnMouseUp(self) -- release button
	self.update:SetScript("OnUpdate",nil)
end

function rematch:MinimapButtonDragUpdate(elapsed)
	local x,y = GetCursorPosition()
	local minX,minY = Minimap:GetLeft(), Minimap:GetBottom()
	local scale = Minimap:GetEffectiveScale()
	settings.minimapButtonPosition = math.deg(math.atan2(y/scale-minY-70,minX-x/scale+70))
	rematch:MinimapButtonPosition()
end

function rematch:MinimapButtonPosition()
	local angle = settings.minimapButtonPosition or -162
	RematchMinimapButton:SetPoint("TOPLEFT",Minimap,"TOPLEFT",52-(80*cos(angle)),(80*sin(angle))-52)
end

function rematch.ReflectSetting.UseMinimapButton()
	if settings.UseMinimapButton then
		rematch:CreateMinimapButton()
		rematch:MinimapButtonPosition()
	end
	if RematchMinimapButton then
		RematchMinimapButton:SetShown(settings.UseMinimapButton)
	end
end

--[[ Rematch Pet Journal Button ]]

function rematch:CreateJournalButton()
	if RematchJournalButton then return end

	-- find the last frame anchored in a chain to the Find Battle button, if any
	local lastAnchor
	local function findAnchor(anchoredTo,...)
		lastAnchor = anchoredTo
		for i=1,select("#",...) do
			local button = select(i,...)
			if select(2,button:GetPoint())==anchoredTo then
				findAnchor(button,PetJournal:GetChildren())
				return
			end
		end
	end
	findAnchor(PetJournalFindBattle,PetJournal:GetChildren())

	-- create the button
	local button = CreateFrame("Button","RematchJournalButton",PetJournal,"UIPanelButtonTemplate") -- MagicButtonTemplate
	button:SetSize(100,22)
	button:SetText("Rematch")
	button:SetScript("OnClick",rematch.Toggle)
	button:SetScript("OnEnter",rematch.ToggleTooltip)
	button:SetScript("OnLeave",rematch.ToggleTooltipHide)

	-- position it anchored to the left of the lastAnchor found above
	button:SetPoint("RIGHT",lastAnchor,"LEFT",-2,0)
	-- now nudge it so it's properly centered if an addon does some yoffset
	local yoff = floor(button:GetBottom())-floor(PetJournal:GetBottom())-4
	button:SetPoint("RIGHT",lastAnchor,"LEFT",-2,-yoff)
end

function rematch.ReflectSetting.HideJournalButton()
	if PetJournal then
		local button = RematchJournalButton
		local hide = settings.HideJournalButton
		if button then
			button:SetShown(not hide)
		elseif not hide and not button then
			rematch:CreateJournalButton()
		end
	end
end

function rematch.ReflectSetting.UseTypeBar()
	rematch:UpdateTypeBar()
end

function rematch.ReflectSetting.WideSearchBar()
	local wide = settings.WideSearchBar
	rematch.browser.toggle:SetShown(not wide)
	rematch.browser.filterButton:SetShown(not wide)
	rematch.browser.smallFilterButton:SetShown(wide)
	rematch.browser.searchBox:SetWidth(wide and 178 or 124)
end

function rematch.ReflectSetting.OnlyBattlePets()
	rematch:UpdateBrowser()
end